﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ivi.Visa.Interop;
using System.Windows;
using System.IO;
using System.Globalization;
using System.Threading;
using System.Windows.Media;
using System.Timers;
using System.Diagnostics;

namespace DAQ
{
    /// <summary>
    /// 
    /// </summary>
    public partial class MainWindow : Window
    {

        /// <summary>
        /// Reset DMM, get instrument ID and show message at DMM display
        /// Revisions:
        ///     2016-02-24 D.A. added:
        ///         - returns logic value indicating whether connection is successful 
        /// </summary>
        /// <param name="Addr"></param>
        public bool InstrumentSetup(string Addr)
        {
            try
            {
                Settings.Dmm.IO = (IMessage)Settings.rm.Open(Addr, AccessMode.NO_LOCK, 2000, ""); //Open up a handle to the DMM 
                Settings.Dmm.IO.Timeout = 2000;

                //First start with a reset state
                Settings.Dmm.IO.Clear();                    //Send a device clear first to stop any measurements in process
                Settings.Dmm.WriteString("*RST", true);     //Reset the device
                Settings.Dmm.WriteString("*IDN?", true);    //Get the IDN string                
                Settings.IDN = Settings.Dmm.ReadString();

                Settings.Dmm.WriteString("DISP OFF", true);
                Settings.Dmm.WriteString("DISP:TEXT \"Connected to DAQ\"", true);   // Write message at DMM display
                return true;
            }
            catch (Exception e)
            {
                MessageBox.Show("Connection failed (" + e.Message + ")", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
        }


        /// <summary>
        /// Configure DMM for selected measurement, acquire and write data to file
        /// (running in separate thread)
        /// Revisions:
        ///     2016-02-24 D.A. added:
        ///         - external trigger support
        ///         - timing calculation for 50Hz / 60Hz AC power
        /// </summary>
        public void DAQmethod()
        {
            Settings.Abort = false;     // not aborted

            try
            {
                Stopwatch stopWatch = new Stopwatch();

                Settings.FileHandle = new StreamWriter(Settings.FileName);

                // Header
                Settings.FileHandle.WriteLine("* Date and time:" + DateTime.Now);
                Settings.FileHandle.WriteLine("* Instrument:" + Settings.IDN);
                if (Settings.ExtTrigger)
                {
                    if (Settings.SampleRate != 0)
                        Settings.FileHandle.WriteLine("* External trigger - Sample rate: " + Settings.SampleRate + " samples/s");
                    else
                        Settings.FileHandle.WriteLine("* External trigger - 'time' should be determined from trigger source");
                }
                else
                {
                    Settings.FileHandle.WriteLine("* Sample rate: " + Settings.SampleRate + " samples/s");
                    Settings.FileHandle.WriteLine("* Recording time: " + Settings.RecTime + " s");
                }

                Settings.FileHandle.WriteLine("* Measurement range: " + Settings.MeasurementType + " - " +
                    Settings.MeasurementRange);
                Settings.FileHandle.WriteLine("* -------- DAQ utility by Davor Antonic --------");
                Settings.FileHandle.WriteLine("*");
                Settings.FileHandle.WriteLine("* Data format: [time value] (Tab delimited)");

                //Configure the DMM measurement state
                // DC voltage/current range selection
                if (Settings.MeasurementType == "VOLT" || Settings.MeasurementType == "CURR")
                {
                    Settings.Dmm.WriteString("CONF:" + Settings.MeasurementType + ":DC " + Settings.MeasurementRange, true);
                }
                else
                {
                    Settings.Dmm.WriteString("CONF:" + Settings.MeasurementType + " " + Settings.MeasurementRange, true);
                }
                if (Settings.MeasurementType == "VOLT")
                {
                    //10MOhm voltage input impedance setting , "ON" sets >10GOhm
                    Settings.Dmm.WriteString(Settings.MeasurementType + ":IMP:AUTO OFF", true);
                }
                if (Settings.MeasurementType == "VOLT" || Settings.MeasurementType == "CURR")
                {
                    Settings.Dmm.WriteString("SENS:" + Settings.MeasurementType + ":DC:ZERO:AUTO OFF", true);   // Autozero Off                
                }
                //Set integration time to .02 PLC (400 us at 50 Hz; 333 us at 60 Hz)
                Settings.Dmm.WriteString(Settings.MeasurementType + ":NPLC .02", true);
                CheckDMMError(Settings.Dmm);                                                // Check the DMM for errors


                // Trigger settings
                int totNoSamples = 0;
                if (Settings.ExtTrigger)
                {
                    // Set trigger to external and set slope
                    if (Settings.UpTrigger) Settings.Dmm.WriteString("TRIG:SOUR EXT;SLOP POS", true);
                    else Settings.Dmm.WriteString("TRIG:SOUR EXT;SLOP NEG", true);

                    Settings.Dmm.WriteString("TRIG:DEL:AUTO OFF", true);            //Turn OFF Automatic trigger delay
                    Settings.Dmm.WriteString("TRIG:DEL 0", true);
                    Settings.Dmm.WriteString("TRIG:COUN INF", true);    //Set trigger count                
                }
                else
                {
                    totNoSamples = Settings.SampleRate * Settings.RecTime;

                    Settings.Dmm.WriteString("TRIG:SOUR IMM", true);                //Set trigger source to internal                                
                    Settings.Dmm.WriteString("TRIG:DEL:AUTO OFF", true);            //Turn OFF Automatic trigger delay
                    // Set trigger delay: T = delay + Integration time + 0.51ms => Delay = T - 0.51ms - 0.02 / MainFreq
                    double delay = 1 / (double)Settings.SampleRate - 0.00051 - 0.02 / Settings.MainFreq;
                    string strDelay = delay.ToString(CultureInfo.InvariantCulture);
                    Settings.Dmm.WriteString("TRIG:DEL " + strDelay, true);
                    // Set trigger count to infinity - requires Firmware version 2.11 or above
                    Settings.Dmm.WriteString("TRIG:COUN INF", true);
                }
                CheckDMMError(Settings.Dmm);                                    // Check the DMM for errors

                // Allow up to 25 seconds for DMM to make readings
                Settings.Dmm.IO.Timeout = 25000;

                // Start acquisition
                Settings.Dmm.WriteString("SYST:BEEP", true);                    // DMM beep
                Settings.Dmm.WriteString("INIT", true);                         // Start
                // Show progress bar
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    if (Settings.ExtTrigger) pbrAcquisition.IsIndeterminate = true;
                    else pbrAcquisition.IsIndeterminate = false;
                    pbrAcquisition.Visibility = Visibility.Visible;
                    tbkAcquisition.Visibility = Visibility.Visible;
                    tbkAcquisition.Text = "";
                    btnStart.IsEnabled = false;
                }));

                double time, value = 0;
                double totalTime = 0;    // beginning time

                // Acquired number of samples
                int noChunks;
                // For external trigger acquisition should be stopped manually
                if (Settings.ExtTrigger) noChunks = Int32.MaxValue;
                else noChunks = totNoSamples / Settings.ChunkSize;

                stopWatch.Start();
                // Acquire requested number of data chunks and check for abort condition
                for (int i = 0; i < noChunks && !Settings.Abort; i++)
                {
                    // Acquire and remove chunk from instrument buffer and save to file
                    Settings.Dmm.WriteString("DATA:REM? " + Settings.ChunkSize.ToString() + ",WAIT", true);
                    //Read the result into a ',' separated array
                    Object[] DCVResult = (object[])Settings.Dmm.ReadList(IEEEASCIIType.ASCIIType_Any, ",;");

                    for (int j = 0; j < Settings.ChunkSize && !Settings.Abort; j++)
                    {
                        if (Settings.SampleRate == 0) time = 0;
                        else time = totalTime + (double)(j + 1) / Settings.SampleRate;
                        value = Convert.ToDouble(DCVResult[j]);
                        // Save chunk to file
                        string line = time.ToString(Settings.SampleRate == 0 ? "F0" : "F6", CultureInfo.InvariantCulture) + "\t" +
                            value.ToString("F6", CultureInfo.InvariantCulture);
                        Settings.FileHandle.WriteLine(line);

                    }
                    Settings.FileHandle.Flush();

                    // update time
                    if (Settings.SampleRate > 0) totalTime += (double)(Settings.ChunkSize) / Settings.SampleRate;

                    // Display progress (for internal trigger remaining time; for external trigger elapsed time)
                    if (!Settings.ExtTrigger)
                    {
                        int remainingTime = Settings.RecTime - (int)totalTime;
                        double progress = 1 - ((double)(remainingTime) / Settings.RecTime);
                        Dispatcher.BeginInvoke(new Action(() =>
                        {
                            pbrAcquisition.Value = 100 * progress;
                            tbkAcquisition.Text = "Remaining time: " + remainingTime.ToString() + " s";
                        }));
                        // Show remaining time at DMM display
                        Settings.Dmm.WriteString("DISP:TEXT \"" + remainingTime.ToString() + " s left\n" +
                            value.ToString("F4") + " " + Settings.MeasurementUnit + "\n\nDAQ by D.A." + "\"", true);
                    }
                    else    // for external trigger display elapsed time
                    {
                        Dispatcher.BeginInvoke(new Action(() =>
                        {
                            tbkAcquisition.Text = String.Format("Time elapsed: {0:hh\\:mm\\:ss}", stopWatch.Elapsed);
                        }));
                        // Show remaining time at DMM display
                        Settings.Dmm.WriteString("DISP:TEXT \"" + String.Format("Elapsed: {0:hh\\:mm\\:ss}", stopWatch.Elapsed) +
                            "\n" + value.ToString("F4") + " " + Settings.MeasurementUnit + "\n\nDAQ by D.A." + "\"", true);
                    }
                }

                // Measurement completed or aborted
                Settings.FileHandle.Close();

                // Double beep
                Settings.Dmm.WriteString("SYST:BEEP", true);
                Thread.Sleep(500);
                Settings.Dmm.WriteString("SYST:BEEP", true);
                Settings.Dmm.WriteString("DISP:TEXT \"DONE\"", true);
                Settings.Dmm.WriteString("*RST", true);                 //Reset DMM
            }
            catch (Exception e)
            {
                Settings.FileHandle.Close();
                MessageBox.Show("Acquisition error (" + e.Message + ")", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            finally
            {
                // Hide progress bar
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    btnStart.Background = Brushes.LightGray;
                    btnStart.IsEnabled = true;
                    pbrAcquisition.Visibility = Visibility.Hidden;
                    tbkAcquisition.Visibility = Visibility.Hidden;
                }));
            }
        }



        /// <summary>
        /// Checks the error queue and throws an exception if error
        /// </summary>
        /// <param name="myDmm"></param>
        public void CheckDMMError(FormattedIO488 myDmm)
        {

            myDmm.WriteString("SYST:ERR?", true);
            string errStr = myDmm.ReadString();

            if (errStr.Contains("No error")) //If no error, then return
                return;
            //If there is an error, read out all of the errors and return them in an exception
            else
            {
                string errStr2 = "";
                do
                {
                    myDmm.WriteString("SYST:ERR?", true);
                    errStr2 = myDmm.ReadString();
                    if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                } while (!errStr2.Contains("No error"));
                throw new Exception("Exception: Encountered system error(s)\n" + errStr);
            }

        }


        /// <summary>
        /// Release DMM resources
        /// </summary>
        public void ReleaseResources()
        {
            // Stop measurement thread
            try { DAQThread.Abort(); }
            catch { }
            Thread.Sleep(1000);

            //Close out your resources
            try { Settings.Dmm.IO.Close(); }
            catch { }
            try { System.Runtime.InteropServices.Marshal.ReleaseComObject(Settings.Dmm); }
            catch { }
            try { System.Runtime.InteropServices.Marshal.ReleaseComObject(Settings.rm); }
            catch { }
            try { Settings.FileHandle.Close(); }
            catch { }
        }
    }
}
