﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;

using Ivi.Visa.Interop;
using System.Diagnostics;
using Microsoft.Win32;
using System.Media;
using System.Threading;
using System.Globalization;
using System.IO;

// ***********************************************************************************
// *  © Davor Antonic, 2016
// *
// * You have a royalty-free right to use, modify, reproduce and distribute
// * the DAQ Application Files (and/or any modified version) in any way
// * you find useful, provided that you agree that Author has no warranty, 
// * obligations or liability for any DAQ Application Files. I will appreciate
// * that you give me the credit if you find usage for the code and also send me 
// * information about bugs you find, derived products etc.
// *
// * In developing DAQ application I used information from David Tu's blog:
// * 'Continuous Measurements with a 34461A Digital Multimeter' available at
// * http://gpete.blogs.keysight.com/2013/11/continuous-measurements-with-34461a.html
// *
// * You may find additional information about DAQ application at 
// * http://antonic.hr/Projects/DAQ/
// ***********************************************************************************

namespace DAQ
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Thread DAQThread;

        public MainWindow()
        {
            InitializeComponent();
        }


        /// <summary>
        /// Set path to 'Config.xml' file in Application Data folder and read settings from XML
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Settings.ConfigFilePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "DAQ\\Config.xml");
            Settings.ReadCommunicationSettings();

            rbAutoTrig.IsChecked = true;    // set default trigger
        }


        /// <summary>
        /// Try to connect to DMM using stored settings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnConnect_Click(object sender, RoutedEventArgs e)
        {
            Settings.rm = new Ivi.Visa.Interop.ResourceManager(); //Open up a new resource manager
            Settings.Dmm = new Ivi.Visa.Interop.FormattedIO488(); //Open a new Formatted IO 488 session 
            try
            {
                //string DmAddr = "TCPIP0::10.10.0.238";
                //string DmAddr = "USB0::0x0957::0x1A07::MY53209449::0::INSTR";
                string DmAddr = "";
                switch (Settings.PreferredConnection)
                {
                    case "USB":
                        DmAddr = Settings.USBconnection;
                        break;
                    case "Ethernet":
                        DmAddr = Settings.EthernetConnection;
                        break;
                    default:    // throw exception here (corrupted data)
                        break;
                }

                if (InstrumentSetup(DmAddr))
                {
                    tbkIDN.Text = Settings.IDN;
                    btnStart.IsEnabled = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error:" + ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }

        }


        /// <summary>
        /// Allow only numerical input for tbxSampleRate and tbxRecTime
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbx_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            double dummy;
            if (!char.IsDigit(e.Text, e.Text.Length - 1))
            //if (!Double.TryParse(e.Text, out dummy))
            {
                e.Handled = true;
            }
        }


        /// <summary>
        /// Data file selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFile_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dlg;
            dlg = new SaveFileDialog();
            dlg.FileName = "Data";      // Default file name
            dlg.DefaultExt = ".txt";    // Default file extension
            dlg.Filter = "Text file (.txt)|*.txt"; // Filter files by extension

            // Process save file dialog box results
            if (dlg.ShowDialog() == true)
            {
                Settings.FileName = dlg.FileName;
                tbxFile.Text = Settings.FileName;
                Settings.newFile = true;
            }

        }


        /// <summary>
        /// Check settings validity (sample rate, recording time, file) and start data acquisition
        /// in separate thread
        /// Revisions:
        ///     2016-02-24 D.A. added:
        ///         - support and validation for external trigger
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            if (rbAutoTrig.IsChecked == true)
            {
                if (tbxSampleRate.Text == "" || tbxRecTime.Text == "" || tbxFile.Text == "" ||
                    !Int32.TryParse(tbxSampleRate.Text, out Settings.SampleRate) ||
                    !Int32.TryParse(tbxRecTime.Text, out Settings.RecTime))
                {
                    MessageBox.Show("Enter valid sample rate, recording time and data file", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
            }
            else
            {
                if (tbxSampleRate.Text != "") Settings.SampleRate = Int32.Parse(tbxSampleRate.Text);
                else Settings.SampleRate = 0;
                Settings.RecTime = 0;
                if (tbxFile.Text == "")
                {
                    MessageBox.Show("Select valid data file", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
            }

            if (Settings.SampleRate > 1000)
            {
                MessageBox.Show("Maximum sample rate is 1000 samples/s", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            if (!Settings.newFile)
            {
                MessageBox.Show("Select new file", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            // Check range selection
            // UPIŠI TO U 'Tag', PRETRAŽUJ KONTROLE KROZ CONTAINER
            Settings.MeasurementType = "";
            Settings.MeasurementRange = "";
            if (rbVoltage.IsChecked == true)
            {
                Settings.MeasurementType = "VOLT";
                if (rb_100mV.IsChecked == true) Settings.MeasurementRange = "0.1";
                if (rb_1V.IsChecked == true) Settings.MeasurementRange = "1";
                if (rb_10V.IsChecked == true) Settings.MeasurementRange = "10";
                if (rb_100V.IsChecked == true) Settings.MeasurementRange = "100";
                if (rb_1000V.IsChecked == true) Settings.MeasurementRange = "1000";
            }

            if (rbCurrent.IsChecked == true)
            {
                Settings.MeasurementType = "VOLT";
                if (rb_100uA.IsChecked == true) Settings.MeasurementRange = "0.0001";
                if (rb_1mA.IsChecked == true) Settings.MeasurementRange = "0.001";
                if (rb_10mA.IsChecked == true) Settings.MeasurementRange = "0.01";
                if (rb_100mA.IsChecked == true) Settings.MeasurementRange = "0.1";
                if (rb_1A.IsChecked == true) Settings.MeasurementRange = "1";
                if (rb_3A.IsChecked == true) Settings.MeasurementRange = "3";
            }

            if (rbResistance2W.IsChecked == true) Settings.MeasurementType = "RES";
            if (rbResistance4W.IsChecked == true) Settings.MeasurementType = "FRES";

            if (rbResistance2W.IsChecked == true || rbResistance4W.IsChecked == true)
            {
                if (rb_100R.IsChecked == true) Settings.MeasurementRange = "100";
                if (rb_1k.IsChecked == true) Settings.MeasurementRange = "1000";
                if (rb_10k.IsChecked == true) Settings.MeasurementRange = "10000";
                if (rb_100k.IsChecked == true) Settings.MeasurementRange = "100000";
                if (rb_1M.IsChecked == true) Settings.MeasurementRange = "1000000";
                if (rb_10M.IsChecked == true) Settings.MeasurementRange = "10000000";
                if (rb_100M.IsChecked == true) Settings.MeasurementRange = "100000000";
            }

            if (Settings.MeasurementType == "" || Settings.MeasurementRange == "")
            {
                MessageBox.Show("Select measuring type and range", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            switch (Settings.MeasurementType)
            {
                case "VOLT":
                    Settings.MeasurementUnit = "V";
                    break;
                case "CURR":
                    Settings.MeasurementUnit = "A";
                    break;
                case "RES":
                case "FRES":
                    Settings.MeasurementUnit = "Ohm";
                    break;
            }

            // Number of samples to get by DATA:REM? (between 1 and 100) - lower values for low sample rates enables 
            // frequent display updates. For external trigger fixed to 20
            if (Settings.ExtTrigger) Settings.ChunkSize = 20;
            else Settings.ChunkSize = Settings.SampleRate <= 100 ? Settings.SampleRate : 100;

            SystemSounds.Beep.Play();
            btnStart.Background = Brushes.PaleGreen;

            // Data acquisition thread
            DAQThread = new Thread(new ThreadStart(DAQmethod));
            // Thread updating GUI should be STA (Single Threaded Apartment)
            DAQThread.SetApartmentState(ApartmentState.STA);
            DAQThread.IsBackground = true;
            DAQThread.Start();

            Settings.newFile = false;
        }

        /// <summary>
        /// Abort data acquisition through shared flag 'Abort'. If set to true aborts main acquisition loop
        /// in DAQmethod(). Already acquired data will be saved to selected file.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            Settings.Abort = true;
        }


        /// <summary>
        /// Show Settings dialog and save settings to Config.xml if dialog was closed with Save (default) button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniSettings_Click(object sender, RoutedEventArgs e)
        {
            SettingsDlg settingsDlg = new SettingsDlg();
            if (settingsDlg.ShowDialog() == true)   // write to Config.xml
            {
                Settings.WriteCommunicationSettings();
            }
        }


        /// <summary>
        /// Close main window (exit application)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniExit_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }


        /// <summary>
        /// Help->About
        /// Set assembly version and license text
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mniAbout_Click(object sender, RoutedEventArgs e)
        {
            About about = new About();
            Version v = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;
            about.tbkVersion.Text = "v " + v.Major + "." + v.Minor + "." + v.Build;
            about.tbxLicense.Text =
                "You have a royalty-free right to use, modify, reproduce and distribute " +
                "the DAQ Application Files (and/or any modified version) in any way " +
                "you find useful, provided that you agree that Author has no warranty, " +
                "obligations or liability for any DAQ Application Files. \n\nI will appreciate " +
                "that you give me the credit if you find usage for the code and also send me " +
                "information about bugs you find, derived products etc.";
            about.ShowDialog();
        }


        /// <summary>
        /// Release resources before exit (main window closing event)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            ReleaseResources();
        }

        /// <summary>
        /// Enable RecTime and SampleRate fields for internal trigger
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbAutoTrig_Checked(object sender, RoutedEventArgs e)
        {
            tbxRecTime.IsEnabled = true;
            tbxRecTime.Text = "";
            tbxSampleRate.Text = "";
            rbUpTrig.IsChecked = false;
            rbDownTrig.IsChecked = false;
            rbUpTrig.IsEnabled = false;
            rbDownTrig.IsEnabled = false;
            Settings.ExtTrigger = false;

            tbxSampleRate.ToolTip = null;
        }

        /// <summary>
        /// Disable RecTime and SampleRate fields for external trigger
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbExtTrig_Checked(object sender, RoutedEventArgs e)
        {
            tbxRecTime.IsEnabled = false;
            tbxRecTime.Text = "";
            tbxSampleRate.Text = "";
            rbUpTrig.IsChecked = true;
            rbUpTrig.IsEnabled = true;
            rbDownTrig.IsEnabled = true;
            Settings.ExtTrigger = true;

            tbxSampleRate.ToolTip = "Set if you know exact frequency of external trigger source, otherwise leave blank";
        }

        private void rbUpTrig_Checked(object sender, RoutedEventArgs e)
        {
            Settings.UpTrigger = true;
        }

        private void rbDownTrig_Checked(object sender, RoutedEventArgs e)
        {
            Settings.UpTrigger = false;
        }

        private void rbVoltage_Checked(object sender, RoutedEventArgs e)
        {
            brdVoltage.Visibility = Visibility.Visible;
            brdCurrent.Visibility = Visibility.Collapsed;
            brdResistance.Visibility = Visibility.Collapsed;
        }

        private void rbCurrent_Checked(object sender, RoutedEventArgs e)
        {
            brdVoltage.Visibility = Visibility.Collapsed;
            brdCurrent.Visibility = Visibility.Visible;
            brdResistance.Visibility = Visibility.Collapsed;
        }

        private void rbResistance_Checked(object sender, RoutedEventArgs e)
        {
            brdVoltage.Visibility = Visibility.Collapsed;
            brdCurrent.Visibility = Visibility.Collapsed;
            brdResistance.Visibility = Visibility.Visible;
        }

    }
}
